@@ -2,6 +2,7 @@ from datetime import datetime
 from unittest.mock import patch
 
 from jdatetime import datetime as jdatetime
+from pytest import raises
 
 # noinspection PyProtectedMember
 from tsetmc import Instrument, _core, get_market_watch_init, \
@@ -14,6 +15,14 @@ def patch_get_content(name):
     return patch.object(_core, 'get_content', lambda _: text)
 
 
+@patch.object(_core, 'get_content')
+def test_get_info_url(mock):
+    with raises(ValueError):
+        Instrument(1).get_info()
+    mock.assert_called_once_with(
+        'http://www.tsetmc.com/tsev2/data/instinfodata.aspx?i=1&c=&e=1')
+
+
 @patch_get_content('fmelli.html')
 def test_get_page_info():
     d = Instrument(1).get_page_info()
@@ -441,3 +450,16 @@ def test_vsadid():
         , 'tno': 0
         , 'tval': 0
         , 'tvol': 0}
+
+
+@patch_get_content('search_firuze.txt')
+def test_from_search_with_numeric_description():
+    # note the "30" in فيروزه - صندوق شاخص30 شركت فيروزه- سهام
+    assert Instrument.from_search('فیروزه').id == 66036975502302203
+
+
+def test_repr():
+    # known ID
+    assert repr(Instrument('IRO1MSMI0001')) == "Instrument('فملی')"
+    # unknown ID
+    assert repr(Instrument(1)) == "Instrument(1)"
