@@ -1,4 +1,4 @@
-__version__ = '0.9.1'
+__version__ = '0.10.0'
 
 from ._core import Instrument, get_market_watch_init, get_closing_price_all, \
-    get_client_type_all, get_key_stats
+    get_client_type_all, get_key_stats, KNOWN_IDS
