@@ -1,7 +1,9 @@
 from functools import partial
 from re import compile as rc
 from datetime import datetime
+from json import load
 from io import BytesIO, StringIO
+from typing import Union
 from ast import literal_eval
 
 
@@ -28,13 +30,15 @@ ALLOWED_MIN_MAX_SEARCH = rc(rf"PSGelStaMax='{F}',PSGelStaMin='{F}").search
 WEAK_YEAR_MIN_MAX_SEARCH = rc(
     rf"MinWeek='{F}',MaxWeek='{F}',MinYear='{F}',MaxYear='{F}'").search
 MONTH_AVG_VOL_SEARCH = rc(r"QTotTran5JAvg='(\d+)'").search
-FIRST_NUMBER_SEARCH = rc(r'\d+').search
 RELATED_COMPANIES = rc(r"var RelatedCompanies=(\[.*\]);").search
 TRADE_HISTORY = rc(r"var TradeHistory=(\[.*\]);").search
 STR_TO_NUM = partial(rc(rf"'({F})'").sub, r'\1')
 INDEX_CHANGE_MATCH = rc(rf"<div class='mn'>(\(?)({F})\)?</div> ({F})%").match
 INDEX_TIMESTAMP_MATCH = rc(r'(\d\d)/(\d+)/(\d+) (\d\d):(\d\d):(\d\d)').match
 
+with open(f'{__file__}/../ids.json', encoding='utf8') as f:
+    KNOWN_IDS: dict[str, str] = load(f)
+
 
 def get_content(url) -> bytes:
     return GET(url).content
@@ -49,13 +53,23 @@ class Instrument:
     # warning/todo:
     # get_page_info and get_inst_info are not tested widely and fail sometimes.
 
-    __slots__ = 'id'
+    __slots__ = 'id', 'isin', 'l13', 'l18'
 
-    def __init__(self, id: int, l30=None):
-        self.id = id
+    def __init__(self, id: Union[int, str]):
+        try:
+            self.id, self.l13, self.l18, self.isin = KNOWN_IDS[f'{id}']
+        except KeyError:
+            if isinstance(id, int):
+                self.id = id
+                self.l13 = self.l18 = self.isin = None
+                return
+            raise KeyError(
+                'id not found in KNOWN_IDS, try Instrument.from_search')
 
     def __repr__(self):
-        return f'Instrument({self.id})'
+        if self.l13 is None:
+            return f'Instrument({self.id})'
+        return f"Instrument('{self.l13}')"
 
     def get_page_info(self) -> dict:
         """Return the static info found on instrument's page.
@@ -107,7 +121,7 @@ class Instrument:
         # return the same response.
         text = get_content(
             f'http://www.tsetmc.com/tsev2/data/instinfodata.aspx'
-            # note that &e=1 parameter is required to get NAV.
+            # &e=1 parameter is required to get NAV
             f'?i={self.id}&c=&e=1').decode()
         # the _s are unknown
         price_info, index_info, orders_info, _, _, _, group_info, _, _ = text.split(';')
@@ -179,8 +193,9 @@ class Instrument:
 
     @staticmethod
     def from_search(s: str) -> 'Instrument':
-        text = fa_norm_text('http://tsetmc.com/tsev2/data/search.aspx?skey=' + s)
-        return Instrument(int(FIRST_NUMBER_SEARCH(text)[0]))
+        """Look up the ID through a web search and return Instrument."""
+        return Instrument(int(get_content(
+            'http://tsetmc.com/tsev2/data/search.aspx?skey=' + s).split(b',', 3)[2]))
 
 
 def get_market_watch_init() -> dict:
